# Basic object support
from OFS.SimpleItem import SimpleItem

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import DTMLFile

from OFS.FindSupport import FindSupport
from AccessControl import ClassSecurityInfo, getSecurityManager

from Perm import Use

import Check
import os

# I think you can actually pull this
# from Globals, but this will work
# just fine ...
dr = os.path.dirname(Check.__file__)
dr = os.path.join(dr, 'rules')
Check.registerRuleDirectory(dr)

class ZChecker(FindSupport, SimpleItem):
    """ A Zope 2 Interface to the checker """
    meta_type = "ZChecker"

    def __init__(self, id, title=""):
        self.id = id
        self.title = title
        self._ignoreList = []

    security = ClassSecurityInfo()

    def _getRules(self):
        """ Rules list is dynamically populated
        we'll need to cache this """
        if not hasattr(self, '_v_RulesList'):
            self._v_RulesList = Check.RulesList()
        return self._v_RulesList

    security.declareProtected(Use, 'checkObjects')
    def ruleMetaTypes(self):
        """ Returns a list of the meta_types we have
        rules for by building a RulesList """
        return self._getRules().meta_types()
        
    security.declareProtected(Use, 'checkObjects')
    def rulesType(self, meta_type):
        """ Returns a list of the rules for a given meta_type """
        r = self._getRules()
        return r.get(meta_type)
 
    security.declareProtected(Use, 'setIgnoreList')
    def setIgnoreList(self, ignore=None, REQUEST=None):
        """ Set the list of functions to ignore """
        if ignore is None: 
            ignore = REQUEST.get('ignore', [])
        assert(Check.isListType(ignore))

        self._ignoreList = ignore
        if REQUEST is not None:
            url = self.absolute_url() + '/manage_alterMessages'
            return REQUEST.RESPONSE.redirect(url)

    security.declareProtected(Use, 'getIgnoreList')
    def getIgnoreList(self):
        """ Return the ignore list """
        return self._ignoreList
    
    security.declareProtected(Use, 'checkObjects')
    def checkObjects(self, objs):
        """ Check the objects this is publicly exposed
        so you can search without using find necessarily
        """
        # form a check object
        c = Check.Check()
        for obj in objs:
            # check each object..
            c.checkObject(obj, self._ignoreList)
        return c.getMsgs()
  
    security.declareProtected(Use, 'processFind')
    def processFind(self, REQUEST=None):
        """ Take the parameters and turn
        them into a bunch of objects, this is
        basically a matter of turning REQUEST into
        a dictionary suitable for passing to Zope Find
        """
        gt = self.REQUEST.get
        kw = {}
        nms = ['obj_ids', 'obj_metatypes', 
               'obj_searchterm', 'obj_expr', 
               'obj_mtime', 'obj_mspec', 
               'obj_permission', 'obj_roles', 
               'search_sub']
        for nm in nms:  
            # build up a keyword list
            kw[nm] = gt(nm, None)

        # results
        res = self.ZopeFind(self.aq_parent, 
                            REQUEST=REQUEST, 
                            **kw)
       
        # for each obj in the res, pass it off to check objects
        # remember that res returns a tuple of object id, object
        # so we need to do list comprehension to get just the object
        return self.checkObjects([ r[1] for r in res ])
   
    # override the FindSupport 
    security.declareProtected(Use, 'manage_findResult')
    manage_findResult = DTMLFile("dtml/find", globals())
    
    security.declareProtected(Use, 'manage_main')
    manage_main = PageTemplateFile("zpt/manage.pt", globals())
    
    security.declareProtected(Use, 'manage_alterMessages')
    manage_alterMessages = PageTemplateFile("zpt/alter.pt", globals())
    
    # show all the options
    manage_options = (
           { 'label':'Manage', 'action':'manage_main'},
           { 'label':'Alter', 'action':'manage_alterMessages'},
           ) + \
           FindSupport.manage_options + \
           SimpleItem.manage_options

manage_addZCheckerForm = PageTemplateFile("zpt/add.pt", globals())

def manage_addZChecker(self, id, title="", REQUEST=None):
    """ Adds in an object """
    self._setObject(id, ZChecker(id, title))
    if REQUEST is not None:
        self.manage_main(self, REQUEST)
